@@ -12,11 +12,11 @@ from typing import (
     Union,
     cast,
 )
+from typing_extensions import Literal
 
 from schema_salad.exceptions import ValidationException
 from schema_salad.sourceline import SourceLine, bullets, strip_dup_lineno
 from schema_salad.utils import json_dumps
-from typing_extensions import Literal
 
 from .errors import WorkflowException
 from .loghandler import _logger
@@ -288,7 +288,9 @@ def static_checker(
             )
             + "\n"
             + SourceLine(sink, "type").makeError(
-                "  with sink '%s' of type %s" % (shortname(sink["id"]), json_dumps(sink["type"]))
+                "  with sink '{}' of type {}".format(
+                    shortname(sink["id"]), json_dumps(sink["type"])
+                )
             )
         )
         if extra_message is not None:
