@@ -17,13 +17,12 @@ from typing import (
     Tuple,
     Union,
 )
-
-from schema_salad.avro.schema import Names
-from schema_salad.ref_resolver import Loader
-from schema_salad.utils import FetcherCallableType
 from typing_extensions import Literal
 
 from ruamel.yaml.comments import CommentedMap
+from schema_salad.avro.schema import Names
+from schema_salad.ref_resolver import Loader
+from schema_salad.utils import FetcherCallableType
 
 from .mpi import MpiConfig
 from .pathmapper import PathMapper
@@ -34,10 +33,10 @@ if TYPE_CHECKING:
     from cwl_utils.parser.cwl_v1_2 import LoadingOptions
 
     from .builder import Builder
+    from .cwlprov.provenance_profile import ProvenanceProfile
+    from .cwlprov.ro import ResearchObject
     from .mutation import MutationManager
     from .process import Process
-    from .provenance import ResearchObject
-    from .provenance_profile import ProvenanceProfile
     from .secrets import SecretStore
     from .software_requirements import DependenciesConfiguration
 
