@@ -2,9 +2,9 @@
 
 import csv
 import datetime
+import json
 import math
 import os
-import re
 import shutil
 import subprocess  # nosec
 import sys
@@ -47,7 +47,7 @@ def _get_docker_machine_mounts() -> List[str]:
                             "-t",
                             "vboxsf",
                         ],
-                        universal_newlines=True,
+                        text=True,
                     ).splitlines()
                 ]
     return __docker_machine_mounts
@@ -83,7 +83,7 @@ class DockerCommandLineJob(ContainerCommandLineJob):
         self,
         builder: Builder,
         joborder: CWLObjectType,
-        make_path_mapper: Callable[..., PathMapper],
+        make_path_mapper: Callable[[List[CWLObjectType], str, RuntimeContext, bool], PathMapper],
         requirements: List[CWLObjectType],
         hints: List[CWLObjectType],
         name: str,
@@ -113,35 +113,18 @@ class DockerCommandLineJob(ContainerCommandLineJob):
             if docker_requirement["dockerImageId"] in _IMAGES:
                 return True
 
-        for line in (
-            subprocess.check_output([self.docker_exec, "images", "--no-trunc", "--all"])  # nosec
-            .decode("utf-8")
-            .splitlines()
-        ):
+        docker_image_id = docker_requirement.get("dockerImageId")
+        if docker_image_id is not None:
             try:
-                match = re.match(r"^([^ ]+)\s+([^ ]+)\s+([^ ]+)", line)
-                split = docker_requirement["dockerImageId"].split(":")
-                if len(split) == 1:
-                    split.append("latest")
-                elif len(split) == 2:
-                    #  if split[1] doesn't  match valid tag names, it is a part of repository
-                    if not re.match(r"[\w][\w.-]{0,127}", split[1]):
-                        split[0] = split[0] + ":" + split[1]
-                        split[1] = "latest"
-                elif len(split) == 3:
-                    if re.match(r"[\w][\w.-]{0,127}", split[2]):
-                        split[0] = split[0] + ":" + split[1]
-                        split[1] = split[2]
-                        del split[2]
-
-                # check for repository:tag match or image id match
-                if match and (
-                    (split[0] == match.group(1) and split[1] == match.group(2))
-                    or docker_requirement["dockerImageId"] == match.group(3)
-                ):
-                    found = True
-                    break
-            except ValueError:
+                manifest = json.loads(
+                    subprocess.check_output(
+                        [self.docker_exec, "inspect", docker_image_id]
+                    ).decode(  # nosec
+                        "utf-8"
+                    )
+                )
+                found = manifest is not None
+            except (OSError, subprocess.CalledProcessError, UnicodeError):
                 pass
 
         if (force_pull or not found) and pull_image:
@@ -180,7 +163,7 @@ class DockerCommandLineJob(ContainerCommandLineJob):
                     )
                     assert loadproc.stdin is not None  # nosec
                     _logger.info("Sending GET request to %s", docker_requirement["dockerLoad"])
-                    req = requests.get(docker_requirement["dockerLoad"], stream=True)
+                    req = requests.get(docker_requirement["dockerLoad"], stream=True, timeout=60)
                     size = 0
                     for chunk in req.iter_content(1024 * 1024):
                         size += len(chunk)
@@ -452,7 +435,7 @@ class PodmanCommandLineJob(DockerCommandLineJob):
         self,
         builder: Builder,
         joborder: CWLObjectType,
-        make_path_mapper: Callable[..., PathMapper],
+        make_path_mapper: Callable[[List[CWLObjectType], str, RuntimeContext, bool], PathMapper],
         requirements: List[CWLObjectType],
         hints: List[CWLObjectType],
         name: str,
