@@ -63,7 +63,9 @@ from .utils import (
 )
 
 if TYPE_CHECKING:
-    from .provenance_profile import ProvenanceProfile  # pylint: disable=unused-import
+    from .cwlprov.provenance_profile import (
+        ProvenanceProfile,  # pylint: disable=unused-import
+    )
 needs_shell_quoting_re = re.compile(r"""(^$|[\s|&;()<>\'"$@])""")
 
 FORCE_SHELLED_POPEN = os.getenv("CWLTOOL_FORCE_SHELL_POPEN", "0") == "1"
@@ -118,7 +120,7 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
         self,
         builder: Builder,
         joborder: CWLObjectType,
-        make_path_mapper: Callable[..., PathMapper],
+        make_path_mapper: Callable[[List[CWLObjectType], str, RuntimeContext, bool], PathMapper],
         requirements: List[CWLObjectType],
         hints: List[CWLObjectType],
         name: str,
@@ -439,7 +441,7 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
             with runtimeContext.workflow_eval_lock:
                 self.output_callback(outputs, processStatus)
 
-        if self.stagedir is not None and os.path.exists(self.stagedir):
+        if runtimeContext.rm_tmpdir and self.stagedir is not None and os.path.exists(self.stagedir):
             _logger.debug(
                 "[job %s] Removing input staging directory %s",
                 self.name,
@@ -508,17 +510,27 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
         # Value must be list rather than integer to utilise pass-by-reference in python
         memory_usage: MutableSequence[Optional[int]] = [None]
 
+        mem_tm: "Optional[Timer]" = None
+
         def get_tree_mem_usage(memory_usage: MutableSequence[Optional[int]]) -> None:
-            children = monitor.children()
+            nonlocal mem_tm
             try:
-                rss = monitor.memory_info().rss
-                while len(children):
-                    rss += sum(process.memory_info().rss for process in children)
-                    children = list(itertools.chain(*(process.children() for process in children)))
-                if memory_usage[0] is None or rss > memory_usage[0]:
-                    memory_usage[0] = rss
+                with monitor.oneshot():
+                    children = monitor.children()
+                    rss = monitor.memory_info().rss
+                    while len(children):
+                        rss += sum(process.memory_info().rss for process in children)
+                        children = list(
+                            itertools.chain(*(process.children() for process in children))
+                        )
+                    if memory_usage[0] is None or rss > memory_usage[0]:
+                        memory_usage[0] = rss
+                mem_tm = Timer(interval=1, function=get_tree_mem_usage, args=(memory_usage,))
+                mem_tm.daemon = True
+                mem_tm.start()
             except psutil.NoSuchProcess:
-                mem_tm.cancel()
+                if mem_tm is not None:
+                    mem_tm.cancel()
 
         mem_tm = Timer(interval=1, function=get_tree_mem_usage, args=(memory_usage,))
         mem_tm.daemon = True
@@ -580,8 +592,9 @@ class CommandLineJob(JobBase):
         env["HOME"] = self.outdir
         env["TMPDIR"] = self.tmpdir
         env["PATH"] = os.environ["PATH"]
-        if "SYSTEMROOT" in os.environ:
-            env["SYSTEMROOT"] = os.environ["SYSTEMROOT"]
+        for extra in ("SYSTEMROOT", "QEMU_LD_PREFIX"):
+            if extra in os.environ:
+                env[extra] = os.environ[extra]
         return env
 
 
@@ -745,16 +758,6 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
                 img_id = str(docker_req["dockerImageId"])
             elif "dockerPull" in docker_req:
                 img_id = str(docker_req["dockerPull"])
-                cmd = [user_space_docker_cmd, "pull", img_id]
-                _logger.info(str(cmd))
-                try:
-                    subprocess.check_call(cmd, stdout=sys.stderr)  # nosec
-                except OSError as exc:
-                    raise SourceLine(docker_req, None, WorkflowException, debug).makeError(
-                        f"Either Docker container {img_id} is not available with "
-                        f"user space docker implementation {user_space_docker_cmd} "
-                        f" or {user_space_docker_cmd} is missing or broken."
-                    ) from exc
             else:
                 raise SourceLine(docker_req, None, WorkflowException, debug).makeError(
                     "Docker image must be specified as 'dockerImageId' or "
@@ -807,7 +810,7 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
                 _logger.debug("%s error", container, exc_info=True)
                 if docker_is_req:
                     raise UnsupportedRequirement(
-                        "{} is required to run this tool: {}".format(container, str(err))
+                        f"{container} is required to run this tool: {str(err)}"
                     ) from err
                 else:
                     raise WorkflowException(
@@ -854,6 +857,10 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
         cid: Optional[str] = None
         while cid is None:
             time.sleep(1)
+            # This is needed to avoid a race condition where the job
+            # was so fast that it already finished when it arrives here
+            if process.returncode is None:
+                process.poll()
             if process.returncode is not None:
                 if cleanup_cidfile:
                     try:
@@ -886,8 +893,8 @@ class ContainerCommandLineJob(JobBase, metaclass=ABCMeta):
         except OSError as exc:
             _logger.warning("Ignored error with %s stats: %s", docker_exe, exc)
             return
-        max_mem_percent: float = 0
-        mem_percent: float = 0
+        max_mem_percent: float = 0.0
+        mem_percent: float = 0.0
         with open(stats_file_name) as stats:
             while True:
                 line = stats.readline()
@@ -1032,6 +1039,26 @@ def _job_popen(
             if sproc.stdin is not None:
                 sproc.stdin.close()
 
+            tm = None
+            if timelimit is not None and timelimit > 0:
+
+                def terminate():  # type: () -> None
+                    try:
+                        _logger.warning(
+                            "[job %s] exceeded time limit of %d seconds and will be terminated",
+                            name,
+                            timelimit,
+                        )
+                        sproc.terminate()
+                    except OSError:
+                        pass
+
+                tm = Timer(timelimit, terminate)
+                tm.daemon = True
+                tm.start()
+            if monitor_function:
+                monitor_function(sproc)
+
             rcode = sproc.wait()
 
             return rcode
