@@ -33,7 +33,10 @@ from typing import (
 
 import argcomplete
 import coloredlogs
-import pkg_resources  # part of setuptools
+import ruamel.yaml
+from importlib_resources import files
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from ruamel.yaml.main import YAML
 from schema_salad.exceptions import ValidationException
 from schema_salad.ref_resolver import Loader, file_uri, uri_file_path
 from schema_salad.sourceline import cmap, strip_dup_lineno
@@ -45,13 +48,17 @@ from schema_salad.utils import (
     yaml_no_ts,
 )
 
-import ruamel.yaml
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
-from ruamel.yaml.main import YAML
-
 from . import CWL_CONTENT_TYPES, workflow
 from .argparser import arg_parser, generate_parser, get_default_args
 from .context import LoadingContext, RuntimeContext, getdefault
+from .cwlprov.ro import ResearchObject  # , WritableBagFile
+from .cwlprov.writablebagfile import (  # change this later
+    WritableBagFile,
+    close_ro,
+    create_job,
+    open_log_file_for_activity,
+    packed_workflow,
+)
 from .cwlrdf import printdot, printrdf
 from .errors import (
     ArgumentException,
@@ -85,7 +92,6 @@ from .process import (
     use_standard_schema,
 )
 from .procgenerator import ProcessGenerator
-from .provenance import ResearchObject, WritableBagFile
 from .resolver import ga4gh_tool_registries, tool_resolver
 from .secrets import SecretStore
 from .software_requirements import (
@@ -639,12 +645,9 @@ def setup_schema(
     if custom_schema_callback is not None:
         custom_schema_callback()
     elif args.enable_ext:
-        with pkg_resources.resource_stream(__name__, "extensions.yml") as res:
-            ext10 = res.read().decode("utf-8")
-        with pkg_resources.resource_stream(__name__, "extensions-v1.1.yml") as res:
-            ext11 = res.read().decode("utf-8")
-        with pkg_resources.resource_stream(__name__, "extensions-v1.2.yml") as res:
-            ext12 = res.read().decode("utf-8")
+        ext10 = files("cwltool").joinpath("extensions.yml").read_text("utf-8")
+        ext11 = files("cwltool").joinpath("extensions-v1.1.yml").read_text("utf-8")
+        ext12 = files("cwltool").joinpath("extensions-v1.2.yml").read_text("utf-8")
         use_custom_schema("v1.0", "http://commonwl.org/cwltool", ext10)
         use_custom_schema("v1.1", "http://commonwl.org/cwltool", ext11)
         use_custom_schema("v1.2", "http://commonwl.org/cwltool", ext12)
@@ -692,7 +695,7 @@ def setup_provenance(
         full_name=args.cwl_full_name,
     )
     runtimeContext.research_obj = ro
-    log_file_io = ro.open_log_file_for_activity(ro.engine_uuid)
+    log_file_io = open_log_file_for_activity(ro, ro.engine_uuid)
     prov_log_handler = logging.StreamHandler(log_file_io)
 
     prov_log_handler.setFormatter(ProvLogFormatter())
@@ -961,7 +964,7 @@ def main(
             if hasattr(sys.stdout, "detach"):
                 stdout = io.TextIOWrapper(sys.stdout.buffer, encoding="utf-8")
             else:
-                stdout = getwriter("utf-8")(sys.stdout)  # type: ignore[assignment,arg-type]
+                stdout = getwriter("utf-8")(sys.stdout)  # type: ignore
         else:
             stdout = sys.stdout
         stdout = cast(IO[str], stdout)
@@ -1099,7 +1102,7 @@ def main(
 
             if args.provenance and runtimeContext.research_obj:
                 # Can't really be combined with args.pack at same time
-                runtimeContext.research_obj.packed_workflow(print_pack(loadingContext, uri))
+                packed_workflow(runtimeContext.research_obj, print_pack(loadingContext, uri))
 
             if args.print_pre:
                 json_dump(
@@ -1298,7 +1301,7 @@ def main(
 
             if out is not None:
                 if runtimeContext.research_obj is not None:
-                    runtimeContext.research_obj.create_job(out, True)
+                    create_job(runtimeContext.research_obj, out, True)
 
                     def remove_at_id(doc: CWLObjectType) -> None:
                         for key in list(doc.keys()):
@@ -1383,7 +1386,9 @@ def main(
         ):
             research_obj = runtimeContext.research_obj
             if loadingContext.loader is not None:
-                research_obj.generate_snapshot(prov_deps(workflowobj, loadingContext.loader, uri))
+                research_obj.generate_snapshot(
+                    prov_deps(cast(CWLObjectType, processobj), loadingContext.loader, uri)
+                )
             else:
                 _logger.warning(
                     "Unable to generate provenance snapshot "
@@ -1401,7 +1406,7 @@ def main(
                 # Why not use prov_log_handler.stream ? That is not part of the
                 # public API for logging.StreamHandler
                 prov_log_handler.close()
-            research_obj.close(args.provenance)
+            close_ro(research_obj, args.provenance)
 
         _logger.removeHandler(stderr_handler)
         _logger.addHandler(defaultStreamHandler)
@@ -1433,7 +1438,8 @@ def windows_check() -> None:
             "Windows Subsystem for Linux 2 (WSL2). If don't need to execute "
             "CWL documents, then you can ignore this warning, but please "
             "consider migrating to https://pypi.org/project/cwl-utils/ "
-            "for your CWL document processing needs."
+            "for your CWL document processing needs.",
+            stacklevel=1,
         )
 
 
