@@ -33,9 +33,10 @@ from typing import (
 )
 
 from cwl_utils import expression
+from importlib_resources import files
 from mypy_extensions import mypyc_attr
-from pkg_resources import resource_stream
 from rdflib import Graph
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.avro.schema import (
     Names,
     Schema,
@@ -49,8 +50,6 @@ from schema_salad.sourceline import SourceLine, strip_dup_lineno
 from schema_salad.utils import convert_to_dict
 from schema_salad.validate import avro_type_name, validate_ex
 
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
-
 from .builder import INPUT_OBJ_VOCAB, Builder
 from .context import LoadingContext, RuntimeContext, getdefault
 from .errors import UnsupportedRequirement, WorkflowException
@@ -59,7 +58,7 @@ from .mpi import MPIRequirementName
 from .pathmapper import MapperEnt, PathMapper
 from .secrets import SecretStore
 from .stdfsaccess import StdFsAccess
-from .update import INTERNAL_VERSION, ORIGINAL_CWLVERSION
+from .update import INTERNAL_VERSION, ORDERED_VERSIONS, ORIGINAL_CWLVERSION
 from .utils import (
     CWLObjectType,
     CWLOutputAtomType,
@@ -80,7 +79,7 @@ from .utils import (
 from .validate_js import validate_js_expressions
 
 if TYPE_CHECKING:
-    from .provenance_profile import ProvenanceProfile
+    from .cwlprov.provenance_profile import ProvenanceProfile
 
 
 class LogAsDebugFilter(logging.Filter):
@@ -195,22 +194,19 @@ def get_schema(
         version = ".".join(version.split(".")[:-1])
     for f in cwl_files:
         try:
-            res = resource_stream(__name__, f"schemas/{version}/{f}")
-            cache["https://w3id.org/cwl/" + f] = res.read().decode("UTF-8")
-            res.close()
+            res = files("cwltool").joinpath(f"schemas/{version}/{f}")
+            cache["https://w3id.org/cwl/" + f] = res.read_text("UTF-8")
         except OSError:
             pass
 
     for f in salad_files:
         try:
-            res = resource_stream(
-                __name__,
+            res = files("cwltool").joinpath(
                 f"schemas/{version}/salad/schema_salad/metaschema/{f}",
             )
-            cache["https://w3id.org/cwl/salad/schema_salad/metaschema/" + f] = res.read().decode(
+            cache["https://w3id.org/cwl/salad/schema_salad/metaschema/" + f] = res.read_text(
                 "UTF-8"
             )
-            res.close()
         except OSError:
             pass
 
@@ -268,7 +264,8 @@ def stage_files(
                     "File staging conflict, trying to stage both %s and %s to the same target %s"
                     % (targets[entry.target].resolved, entry.resolved, entry.target)
                 )
-
+    # refresh the items, since we may have updated the pathmapper due to file name clashes
+    items = pathmapper.items() if not symlink else pathmapper.items_exclude_children()
     for key, entry in items:
         if not entry.staged:
             continue
@@ -519,12 +516,17 @@ def eval_resource(
 ) -> Optional[Union[str, int, float]]:
     if isinstance(resource_req, str) and expression.needs_parsing(resource_req):
         result = builder.do_eval(resource_req)
+        if isinstance(result, float):
+            if ORDERED_VERSIONS.index(builder.cwlVersion) >= ORDERED_VERSIONS.index("v1.2.0-dev4"):
+                return result
+            raise WorkflowException(
+                "Floats are not valid in resource requirement expressions prior "
+                f"to CWL v1.2: {resource_req} returned {result}."
+            )
         if isinstance(result, (str, int)) or result is None:
             return result
         raise WorkflowException(
-            "Got incorrect return type {} from resource expression evaluation of {}.".format(
-                type(result), resource_req
-            )
+            f"Got incorrect return type {type(result)} from resource expression evaluation of {resource_req}."
         )
     return resource_req
 
@@ -978,17 +980,20 @@ hints:
         ):
             if rsc is None:
                 continue
-            mn = mx = None  # type: Optional[Union[int, float]]
+            mn: Optional[Union[int, float]] = None
+            mx: Optional[Union[int, float]] = None
             if rsc.get(a + "Min"):
-                mn = cast(
-                    Union[int, float],
-                    eval_resource(builder, cast(Union[str, int, float], rsc[a + "Min"])),
-                )
+                with SourceLine(rsc, f"{a}Min", WorkflowException, runtimeContext.debug):
+                    mn = cast(
+                        Union[int, float],
+                        eval_resource(builder, cast(Union[str, int, float], rsc[a + "Min"])),
+                    )
             if rsc.get(a + "Max"):
-                mx = cast(
-                    Union[int, float],
-                    eval_resource(builder, cast(Union[str, int, float], rsc[a + "Max"])),
-                )
+                with SourceLine(rsc, f"{a}Max", WorkflowException, runtimeContext.debug):
+                    mx = cast(
+                        Union[int, float],
+                        eval_resource(builder, cast(Union[str, int, float], rsc[a + "Max"])),
+                    )
             if mn is None:
                 mn = mx
             elif mx is None:
