@@ -1,5 +1,6 @@
 """Shared functions and other definitions."""
 import collections
+import fcntl
 import os
 import random
 import shutil
@@ -16,12 +17,12 @@ from functools import partial
 from itertools import zip_longest
 from pathlib import Path, PurePosixPath
 from tempfile import NamedTemporaryFile
-from types import ModuleType
 from typing import (
     IO,
     TYPE_CHECKING,
     Any,
     Callable,
+    Deque,
     Dict,
     Generator,
     Iterable,
@@ -35,15 +36,19 @@ from typing import (
     Union,
     cast,
 )
+from typing_extensions import Literal, TypedDict
 
-import pkg_resources
 import requests
 from cachecontrol import CacheControl
 from cachecontrol.caches import FileCache
-from mypy_extensions import TypedDict, mypyc_attr
+from mypy_extensions import mypyc_attr
 from schema_salad.exceptions import ValidationException
 from schema_salad.ref_resolver import Loader
-from typing_extensions import Deque, Literal
+
+if sys.version_info >= (3, 8):
+    import importlib.metadata as importlib_metadata
+else:
+    import importlib_metadata
 
 if TYPE_CHECKING:
     from .command_line_tool import CallbackJob, ExpressionJob
@@ -97,14 +102,15 @@ DirectoryType = TypedDict(
 )
 JSONAtomType = Union[Dict[str, Any], List[Any], str, int, float, bool, None]
 JSONType = Union[Dict[str, JSONAtomType], List[JSONAtomType], str, int, float, bool, None]
-WorkflowStateItem = NamedTuple(
-    "WorkflowStateItem",
-    [
-        ("parameter", CWLObjectType),
-        ("value", Optional[CWLOutputType]),
-        ("success", str),
-    ],
-)
+
+
+class WorkflowStateItem(NamedTuple):
+    """Workflow state item."""
+
+    parameter: CWLObjectType
+    value: Optional[CWLOutputType]
+    success: str
+
 
 ParametersType = List[CWLObjectType]
 StepType = CWLObjectType  # WorkflowStep
@@ -114,9 +120,9 @@ LoadListingType = Union[Literal["no_listing"], Literal["shallow_listing"], Liter
 
 def versionstring() -> str:
     """Version of CWLtool used to execute the workflow."""
-    pkg = pkg_resources.require("cwltool")
+    pkg = importlib_metadata.version("cwltool")
     if pkg:
-        return f"{sys.argv[0]} {pkg[0].version}"
+        return f"{sys.argv[0]} {pkg}"
     return "{} {}".format(sys.argv[0], "unknown version")
 
 
@@ -239,29 +245,12 @@ def random_outdir() -> str:
     return __random_outdir
 
 
-#
-# Simple multi-platform (fcntl/msvrt) file locking wrapper
-#
-fcntl: Optional[ModuleType] = None
-msvcrt: Optional[ModuleType] = None
-try:
-    import fcntl
-except ImportError:
-    import msvcrt
-
-
 def shared_file_lock(fd: IO[Any]) -> None:
-    if fcntl:
-        fcntl.flock(fd.fileno(), fcntl.LOCK_SH)
-    elif msvcrt:
-        msvcrt.locking(fd.fileno(), msvcrt.LK_LOCK, 1024)
+    fcntl.flock(fd.fileno(), fcntl.LOCK_SH)
 
 
 def upgrade_lock(fd: IO[Any]) -> None:
-    if fcntl:
-        fcntl.flock(fd.fileno(), fcntl.LOCK_EX)
-    elif msvcrt:
-        pass
+    fcntl.flock(fd.fileno(), fcntl.LOCK_EX)
 
 
 def adjustFileObjs(rec: Any, op: Union[Callable[..., Any], "partial[Any]"]) -> None:
