@@ -15,6 +15,7 @@ import pydot
 import pytest
 from cwl_utils.errors import JavascriptException
 from cwl_utils.sandboxjs import param_re
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.exceptions import ValidationException
 
 import cwltool.checker
@@ -28,7 +29,6 @@ from cwltool.errors import WorkflowException
 from cwltool.main import main
 from cwltool.process import CWL_IANA
 from cwltool.utils import CWLObjectType, dedup
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 from .util import get_data, get_main_output, needs_docker, working_directory
 
@@ -816,8 +816,7 @@ def test_lifting() -> None:
     # fails if the step 'out' doesn't match.
     factory = cwltool.factory.Factory()
     with pytest.raises(ValidationException):
-        echo = factory.make(get_data("tests/test_bad_outputs_wf.cwl"))
-        assert echo(inp="foo") == {"out": "foo\n"}
+        factory.make(get_data("tests/test_bad_outputs_wf.cwl"))
 
 
 def test_malformed_outputs() -> None:
@@ -1474,7 +1473,9 @@ def test_bad_userspace_runtime(factor: str) -> None:
     )
     error_code, stdout, stderr = get_main_output(commands)
     stderr = re.sub(r"\s\s+", " ", stderr)
-    assert "or quaquioN is missing or broken" in stderr, stderr
+    assert ("or quaquioN is missing or broken" in stderr) or (
+        "No such file or directory: 'quaquioN'" in stderr
+    ), stderr
     assert error_code == 1
 
 
@@ -1799,3 +1800,61 @@ def test_validate_optional_src_with_mandatory_sink() -> None:
     stderr = re.sub(r"\s\s+", " ", stderr)
     assert 'Source \'opt_file\' of type ["null", "File"] may be incompatible' in stderr
     assert "with sink 'r' of type \"File\"" in stderr
+
+
+def test_res_req_expr_float_1_0() -> None:
+    """Confirm expected error when returning a float value from a ResourceRequirement expr in CWL v1.0."""
+    exit_code, stdout, stderr = get_main_output(
+        [
+            get_data("tests/wf/resreq_expr_float_v1_0.cwl"),
+            "--input_bam",
+            get_data("tests/wf/whale.txt"),
+        ]
+    )
+    assert exit_code == 1
+    stderr = re.sub(r"\s\s+", " ", stderr)
+    assert "Floats are not valid in resource requirement expressions" in stderr
+    assert "prior to CWL v1.2" in stderr
+    assert "$((2 * inputs.input_bam.size) / 3.14159) returned" in stderr
+
+
+def test_res_req_expr_float_1_2() -> None:
+    """Confirm no error when returning a float value from a ResourceRequirement expr in CWL v1.0."""
+    exit_code, stdout, stderr = get_main_output(
+        [
+            get_data("tests/wf/resreq_expr_float_v1_2.cwl"),
+            "--input_bam",
+            get_data("tests/wf/whale.txt"),
+        ]
+    )
+    assert exit_code == 0, stderr
+    assert json.loads(stdout)["result"]["outdirSize"] >= 708
+    assert json.loads(stdout)["result"]["tmpdirSize"] >= 708
+
+
+def test_very_small_and_large_floats() -> None:
+    """Confirm that very small or large numbers are not transformed into scientific notation."""
+    exit_code, stdout, stderr = get_main_output(
+        [
+            get_data("tests/wf/floats_small_and_large.cwl"),
+        ]
+    )
+    assert exit_code == 0, stderr
+    assert json.loads(stdout)["result"] == "0.00001 0.0000123 123000 1230000"
+
+
+def test_invalid_nested_array() -> None:
+    """Test feature proposed for CWL v1.3 in a CWL v1.2 document."""
+    exit_code, stdout, stderr = get_main_output(
+        [
+            "--validate",
+            get_data("tests/nested-array.cwl"),
+        ]
+    )
+    assert exit_code == 1, stderr
+    stderr = re.sub(r"\n\s+", " ", stderr)
+    stderr = re.sub(r"\s\s+", " ", stderr)
+    assert "Tool definition failed validation:" in stderr
+    assert (
+        "tests/nested-array.cwl:6:5: Field 'type' references unknown identifier 'string[][]'"
+    ) in stderr
