@@ -1,10 +1,12 @@
 """Test that all temporary directories respect the --tmpdir-prefix and --tmp-outdir-prefix options."""
+import re
 import subprocess
 import sys
 from pathlib import Path
 from typing import List, cast
 
 import pytest
+from ruamel.yaml.comments import CommentedMap
 from schema_salad.avro import schema
 from schema_salad.sourceline import cmap
 
@@ -14,13 +16,12 @@ from cwltool.context import LoadingContext, RuntimeContext
 from cwltool.docker import DockerCommandLineJob
 from cwltool.job import JobBase
 from cwltool.main import main
-from cwltool.pathmapper import MapperEnt, PathMapper
+from cwltool.pathmapper import MapperEnt
 from cwltool.stdfsaccess import StdFsAccess
 from cwltool.update import INTERNAL_VERSION, ORIGINAL_CWLVERSION
 from cwltool.utils import create_tmp_dir
-from ruamel.yaml.comments import CommentedMap
 
-from .util import get_data, needs_docker
+from .util import get_data, get_main_output, needs_docker
 
 
 def test_docker_commandLineTool_job_tmpdir_prefix(tmp_path: Path) -> None:
@@ -144,7 +145,9 @@ def test_dockerfile_tmpdir_prefix(tmp_path: Path, monkeypatch: pytest.MonkeyPatc
         INTERNAL_VERSION,
         "docker",
     )
-    assert DockerCommandLineJob(builder, {}, PathMapper, [], [], "").get_image(
+    assert DockerCommandLineJob(
+        builder, {}, CommandLineTool.make_path_mapper, [], [], ""
+    ).get_image(
         {
             "class": "DockerRequirement",
             "dockerFile": "FROM debian:stable-slim",
@@ -193,7 +196,7 @@ def test_docker_tmpdir_prefix(tmp_path: Path) -> None:
         INTERNAL_VERSION,
         "docker",
     )
-    job = DockerCommandLineJob(builder, {}, PathMapper, [], [], "")
+    job = DockerCommandLineJob(builder, {}, CommandLineTool.make_path_mapper, [], [], "")
     runtime: List[str] = []
 
     volume_writable_file = MapperEnt(
@@ -271,3 +274,22 @@ def test_remove_tmpdirs(tmp_path: Path) -> None:
         == 0
     )
     assert len(list(tmp_path.iterdir())) == 0
+
+
+def test_leave_tmpdirs(tmp_path: Path) -> None:
+    """Test that the tmpdirs including input staging directories are retained after the job execution."""
+    error_code, stdout, stderr = get_main_output(
+        [
+            "--debug",
+            "--tmpdir-prefix",
+            str(f"{tmp_path}/tmp/"),
+            "--leave-tmpdir",
+            "--outdir",
+            str(f"{tmp_path}/out/"),
+            get_data("tests/env4.cwl"),
+        ]
+    )
+    assert error_code == 0
+    assert re.search(rf"\"{re.escape(str(tmp_path))}/tmp/.*/env0\.py\"", stderr)
+    assert len(list((tmp_path / "tmp").iterdir())) == 3
+    assert len(list((tmp_path / "tmp").glob("**/env0.py"))) == 1
